@@ -69,6 +69,10 @@ NULL
 #'   \item{\code{size()}}{
 #'     Returns the number of items in the map.
 #'   }
+#'   \item{\code{clone()}}{
+#'     Returns a copy of the fastmap object. This is a shallow clone; objects in
+#'     the fastmap will not be copied.
+#'   }
 #'   \item{\code{as_list(sort = FALSE)}}{
 #'     Return a named list where the names are the keys from the map, and the
 #'     values are the values. By default, the keys will be in arbitrary order.
@@ -167,7 +171,11 @@ fastmap <- function(missing_default = NULL) {
   # entry is replaced with NA, and n_holes is updated to reflect it; this is
   # instead of simply shrinking the holes vector, because that involves copying
   # the entire object.
+  # Has a structure like this, with numbers at the beginning and NA's at the
+  # end:
+  #   1 2 3 4 5 NA NA NA
   holes <- NULL
+  # A number. For the example above, n_holes would be 5.
   n_holes <- NULL
 
   # ===================================
@@ -313,10 +321,10 @@ fastmap <- function(missing_default = NULL) {
 
   remove <- function(keys) {
     if (!(is.character(keys) || is.null(keys))) {
-      stop("mget: `keys` must be a character vector or NULL")
+      stop("remove: `keys` must be a character vector or NULL")
     }
     if (any(keys == "") || any(is.na(keys))) {
-      stop('mget: `keys` must not be "" or NA')
+      stop('remove: `keys` must not be "" or NA')
     }
     if (length(keys) == 1) {
       # In the common case of only one key, it's faster to avoid vapply.
@@ -331,19 +339,60 @@ fastmap <- function(missing_default = NULL) {
   }
 
   keys <- function(sort = FALSE) {
-    ensure_restore_map()
-    .Call(C_map_keys, key_idx_map, sort)
+    # The implementation below is equivalent to the following, but about 100x
+    # faster:
+    #   ensure_restore_map()
+    #   .Call(C_map_keys, key_idx_map, sort)
+
+    k <- keys_[non_hole_idxs_()]
+    if (sort) {
+      k <- sort(k, method = "radix")
+    }
+    k
+  }
+
+  clone <- function() {
+    m <- fastmap()
+    # Use this env to access internal objects of the new fastmap object.
+    e <- environment(m$get)
+    e$ensure_restore_map()
+
+    e$n           <- n
+    e$key_idx_map <- .Call(C_map_copy, key_idx_map)
+    e$keys_       <- keys_
+    e$values      <- values
+    e$holes       <- holes
+    e$n_holes     <- n_holes
+
+    m
   }
 
   as_list <- function(sort = FALSE) {
-    ensure_restore_map()
-    keys_idxs <- .Call(C_map_keys_idxs, key_idx_map, sort)
-    result <- values[keys_idxs]
-    names(result) <- names(keys_idxs)
+    idxs <- non_hole_idxs_()
+    result <- values[idxs]
+    names(result) <- keys_[idxs]
+    if (sort) {
+      result <- result[order(names(result))]
+    }
     result
   }
 
 
+  # Internal function
+  #
+  # Returns a vector containing the indices of the non-holes in the keys_ and
+  # values lists.
+  non_hole_idxs_ <- function() {
+    # Need to special case when 0 holes, because x[-integer(0)] (as done in
+    # the other code path) returns an empty vector, instead of the original x.
+    if (n_holes == 0) {
+      seq_along(keys_)
+    } else {
+      h <- holes[seq_len(n_holes)]
+      seq_along(keys_)[-h]
+    }
+  }
+
   # Internal function
   grow <- function() {
     old_values_length <- length(values)
@@ -413,8 +462,7 @@ fastmap <- function(missing_default = NULL) {
 
     # Repopulate key_idx_map.
     key_idx_map <<- .Call(C_map_create)
-    holes <- holes[seq_len(n_holes)]
-    idxs <- seq_along(keys_)[-holes]
+    idxs <- non_hole_idxs_()
     for (idx in idxs) {
       .Call(C_map_set, key_idx_map, keys_[idx], idx)
     }
@@ -430,6 +478,7 @@ fastmap <- function(missing_default = NULL) {
     remove = remove,
     keys = keys,
     size = size,
+    clone = clone,
     as_list = as_list
   )
 }
